﻿using System;
using TcHmiSrv;
using TcHmiSrv.Management;
using Newtonsoft.Json;
using DatabaseManipulation;
using System.Collections.Generic;

namespace DataManagementExtension
{
    // The default class of the server extension must inherit from the IExtension interface.
    // The path of the server extension is available via the ExtensionPath property if the default class of the server extension inherits from the ExtensionWithPath base class instead.
    public class DataManagementExtension : IExtension
    {
        private ITcHmiSrvExtHost _host = null;

        private object _shutdownLock = new object();
        private bool _isShuttingDown = false;

        private TcHmiSrvRequestListener _requestListener = new TcHmiSrvRequestListener();
        private TcHmiSrvShutdownListener _shutdownListener = new TcHmiSrvShutdownListener();

        private Log _logger = null;
        private Data _data = new Data();
        private DatabaseClass databaseClass = new DatabaseClass();


        private Random _rand = new Random("DataManagementExtension".GetHashCode());

        // Initializes the server extension.
        public ErrorValue Init(ITcHmiSrvExtHost host, Context context)
        {
            _logger = new Log(host);

            try
            {
                _host = host;

                // Add event handlers
                _requestListener.OnRequest += OnRequest;
                _shutdownListener.OnShutdown += OnShutdown;

                // Register listeners
                _host.register_listener(context, _requestListener);
                _host.register_listener(context, _shutdownListener);

                _logger.Send(context, "MESSAGE_INIT");
                return ErrorValue.HMI_SUCCESS;
            }
            catch (Exception ex)
            {
                _logger.Send(context, "ERROR_INIT", ex.Message, Severity.Severity_Error);
                return ErrorValue.HMI_E_EXTENSION_LOAD;
            }
        }

        // Called when a client requests a symbol from the extension domain.
        private ErrorValue OnRequest(object sender, TcHmiSrvRequestListener.OnRequestEventArgs e)
        {
            ErrorValue ret = ErrorValue.HMI_SUCCESS;
            Context context = e.Context;
            CommandGroup commands = e.Commands;

            try
            {
                commands.Result = ExtensionErrorValue.HMI_EXT_SUCCESS;
                string mapping = "";

                foreach (Command command in commands)
                {
                    mapping = command.Mapping;

                    try
                    {
                        // Use the mapping to check which command is requested
                        switch (mapping)
                        {
                            case "LoadFromDatabase":
                                ret = LoadFromDatabase(command);
                                break;

                            case "EditFromDatabase":
                                ret = EditFromDatabase(command);
                                break;
                            case "DeleteFromDatabase":
                                ret = DeleteFromDatabase(command);
                                break;
                            case "SaveToDatabase":
                                ret = SaveToDatabase(command);
                                break;

                            case "EmployeeData":
                                ret = EmployeeData(command);
                                break;
                            case "ProductData":
                                ret = ProductData(command);
                                break;
                            case "StoreData":
                                ret = StoreData(command);
                                break;
                            case "StroProductData":
                                ret = StroProductData(command);
                                break;

                            case "ComboString":
                                ret = ComboString(command);
                                break;
                            case "ComboIndex":
                                ret = ComboIndex(command);
                                break;
                            default:
                                ret = ErrorValue.HMI_E_EXTENSION;
                                break;
                        }

                        //if (ret != ErrorValue.HMI_SUCCESS)
                        // Do something on error
                    }
                    catch (Exception ex)
                    {
                        command.ExtensionResult = ExtensionErrorValue.HMI_EXT_FAIL;
                        command.ResultString = _logger.Localize(context, "ERROR_CALL_COMMAND", new string[] { mapping, ex.Message });
                    }
                }
            }
            catch
            {
                commands.Result = ExtensionErrorValue.HMI_EXT_FAIL;
            }
            finally
            {
                if (commands.Result != ExtensionErrorValue.HMI_EXT_SUCCESS)
                {
                    // Reset the read value of the commands to prevent the server from sending invalid data
                    foreach (Command command in commands)
                    {
                        command.ReadValue = null;
                    }

                    ret = ErrorValue.HMI_E_EXTENSION;
                }
            }

            return ret;
        }

        // Called when the server is shutting down. After exiting this function the extension dll will be unloaded.
        private ErrorValue OnShutdown(object sender, TcHmiSrvShutdownListener.OnShutdownEventArgs e)
        {
            // If the extension does not shutdown after 10 seconds (blocking threads) OnShutdown will be called again
            lock (_shutdownLock)
            {
                if (_isShuttingDown)
                {
                    return ErrorValue.HMI_SUCCESS;
                }

                _isShuttingDown = true;

                Context context = e.Context;

                try
                {
                    // Unregister listeners
                    _host.unregister_listener(context, _requestListener);
                    _host.unregister_listener(context, _shutdownListener);

                    _logger.Send(context, "MESSAGE_SHUTDOWN");
                    return ErrorValue.HMI_SUCCESS;
                }
                catch (Exception ex)
                {
                    _logger.Send(context, "ERROR_SHUTDOWN", ex.Message, Severity.Severity_Error);
                    return ErrorValue.HMI_E_EXTENSION;
                }
            }
        }


        private ErrorValue EmployeeData(Command command)
        {
            command.ReadValue = _data.EmployeeData;
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }
        private ErrorValue ProductData(Command command)
        {
            command.ReadValue = _data.ProductData;
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }
        private ErrorValue StoreData(Command command)
        {
            command.ReadValue = _data.StoreData;
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }
        private ErrorValue StroProductData(Command command)
        {
            command.ReadValue = _data.StroProductData;
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }

        private ErrorValue ComboString(Command command)
        {
            command.ReadValue = _data.ComboString;
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }
        private ErrorValue ComboIndex(Command command)
        {
            if (command.WriteValue.IsSet && command.WriteValue.Type == TcHmiSrv.ValueType.ValueType_Int)
            {
                _data.ComboIndex = command.WriteValue;
                List<string> authors = new List<string>();
                for (int i = 0; i < _data.ComboIndex; i++)
                {
                    authors.Add((i + 1).ToString());
                }
                _data.ComboString = JsonConvert.SerializeObject(authors);
            }
            command.ReadValue = _data.ComboIndex;
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }

        private ErrorValue LoadFromDatabase(Command command)
        {

            if (command.WriteValue.IsSet && command.WriteValue.Type == TcHmiSrv.ValueType.ValueType_String)
            {
                try
                {
                    DatabaseOperation op = DatabaseOperation.Load;
                    databaseClass.LoadFromDatabase(op);
                    _data.EmployeeData = databaseClass.EmployeeListJson;
                    _data.ProductData = databaseClass.ProductlistJson;
                    _data.StoreData = databaseClass.StoreListJson;
                    _data.StroProductData = databaseClass.ProductStoreJson;
                    _data.ComboString = databaseClass.ComboString;
                }
                catch (Exception ex) {}
            }


            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }

        private ErrorValue EditFromDatabase(Command command)
        {
            if (command.WriteValue.IsSet && command.WriteValue.Type == TcHmiSrv.ValueType.ValueType_String)
            {
                DatabaseOperation op = DatabaseOperation.Edit;
                _data.StringForDatabase = command.WriteValue;
                databaseClass.EditInDatabase(_data.StringForDatabase, op);
            }
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }



        private ErrorValue DeleteFromDatabase(Command command)
        {
            if (command.WriteValue.IsSet && command.WriteValue.Type == TcHmiSrv.ValueType.ValueType_String)
            {
                _data.StringForDatabase = command.WriteValue;
                DatabaseOperation op = DatabaseOperation.Delete;
                databaseClass.DeleteFromDatabase(_data.StringForDatabase, op);
            }
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }
        private ErrorValue SaveToDatabase(Command command)
        {
            if (command.WriteValue.IsSet && command.WriteValue.Type == TcHmiSrv.ValueType.ValueType_String)
            {
                _data.StringForDatabase = command.WriteValue;
                DatabaseOperation op = DatabaseOperation.Save;
                databaseClass.SaveToDatabase(_data.StringForDatabase, op);
            }
            command.ReadValue = _data.StringForDatabase;
            command.ExtensionResult = ExtensionErrorValue.HMI_EXT_SUCCESS;
            return ErrorValue.HMI_SUCCESS;
        }
    }
}
